@@ -357,10 +357,6 @@ static void cfg_save(config_obj *cfgo)
 	{
 		FILE *fp = fopen(cfgo->url, "w");
 		if(!fp) return;
-		fprintf(fp, "# The GMPC Config file\n"\
-				"# Lines starting with '#' '/' and ';' are coments, and ignored\n"\
-				"# Also empty lines are ignored.\n"\
-				"# User comments will get lost\n");
 		cfg_save_category(cfgo,cfgo->root, fp);	
 		fclose(fp);
 #ifndef WIN32
@@ -657,6 +653,11 @@ static void __int_cfg_set_single_value_as_string(config_obj *cfg, char *class, c
 		cfg_add_child(node,newnode);
 
 	}	
+	else if(strlen(newnode->value) == strlen(value) && !memcmp(newnode->value, value,strlen(newnode->value)))
+	{
+		/* Check if the content is the same, if it is, do nothing */
+		return;
+	}
 	newnode->type = TYPE_ITEM;
 	if(newnode->value){
 		cfg->total_size-= strlen(newnode->value);
@@ -722,6 +723,10 @@ void cfg_set_multiple_value_as_string(config_obj *cfg, char *class, char *key, c
 	cur = cfg_get_multiple_value(cfg, class,key,id);
 	if(cur != NULL)
 	{
+		if(strlen(cur->value) == strlen(value) && !memcmp(cur->value, value, strlen(cur->value)))
+		{
+			return;
+		}
 		if(cur->value){
 			cfg->total_size -= strlen(cur->value);
 			cfg_free_string(cur->value);
@@ -729,6 +734,7 @@ void cfg_set_multiple_value_as_string(config_obj *cfg, char *class, char *key, c
 
 		cur->value = g_strdup(value);
 		cfg->total_size += strlen(cur->value);
+		cfg_save(cfg);
 	}
 	else {
 		config_node *node = cfg_get_single_value(cfg,class,key);
