@@ -40,12 +40,12 @@
 #include "main.h"
 
 /* as internall plugin */
-#include "playlist3-file-browser.h"
-#include "playlist3-find2-browser.h"
-#include "playlist3-find3-browser.h"
-#include "playlist3-tag-browser.h"
-#include "playlist3-artist-browser.h"
-#include "playlist3-current-playlist-browser.h"
+#include "browsers/playlist3-file-browser.h"
+#include "browsers/playlist3-find2-browser.h"
+#include "browsers/playlist3-find3-browser.h"
+#include "browsers/playlist3-tag-browser.h"
+#include "browsers/playlist3-artist-browser.h"
+#include "browsers/playlist3-current-playlist-browser.h"
 
 #include "id3info.h"
 
@@ -62,18 +62,12 @@
  * blub
  */
 #ifndef WIN32
-#include "bacon-message-connection.h"
+#include "bacon/bacon-message-connection.h"
 static BaconMessageConnection *bacon_connection = NULL;
 #endif
 
-#ifdef DEBUG
-/* Testing */
-#include <sys/time.h>
-
-struct timeval tv_old = {0,0};
-#endif
-
 GmpcConnection *gmpcconn = NULL;
+GmpcProfiles *gmpc_profiles = NULL;
 int gmpc_connected = FALSE;
 int gmpc_failed_tries = 0;
 
@@ -116,7 +110,6 @@ guint autoconnect_timeout = 0;
  * The Config object
  */
 config_obj *config = NULL;
-config_obj *profiles = NULL;
 
 /*
  * The Connection object
@@ -203,6 +196,9 @@ int main (int argc, char **argv)
     int i;
     int clean_config = FALSE;
     char *config_path = NULL;
+#ifdef WIN32
+	gchar *packagedir;
+#endif
 #ifdef ENABLE_MMKEYS
     MmKeys *keys = NULL;
 #endif
@@ -259,10 +255,10 @@ int main (int argc, char **argv)
             else if (!strcasecmp(argv[i], _("--version")))
             {
                 printf(("Gnome Music Player Client\n"));
-                printf(_("Version: %s\n"), VERSION);
+                printf(_("Version:\t%s\n"), VERSION);
                 if(revision && revision[0] != '\0')
                 {
-                    printf(_("Revision: %s\n"),revision);
+                    printf(_("Revision:\t%s\n"),revision);
                 }
                 exit(0);
             }
@@ -301,6 +297,10 @@ int main (int argc, char **argv)
 
     }
 
+    /** Check if threading is supported. */	
+    /** initialize it */
+    if(!g_thread_supported())g_thread_init (NULL);
+
     /*
      * initialize gtk
      */
@@ -338,9 +338,9 @@ int main (int argc, char **argv)
         }	
 
     }	
-    /** Check if threading is supported. */	
-    /** initialize it */
-    if(!g_thread_supported())g_thread_init (NULL);
+
+
+
 
     create_gmpc_paths();
 
@@ -364,7 +364,7 @@ int main (int argc, char **argv)
      */
     if(!config_path)
     {
-        url = g_strdup_printf("%s/.gmpc/gmpc.cfg", g_get_home_dir());
+        url = g_strdup_printf("%s%c.gmpc%cgmpc.cfg", g_get_home_dir(),G_DIR_SEPARATOR,G_DIR_SEPARATOR);
     }
     else{
         url = config_path;
@@ -389,26 +389,25 @@ int main (int argc, char **argv)
      * cleanup 
      */
     q_free(url);
-    /**
-     * Profile file
-     */
-    url = cfg_get_single_value_as_string(config, "connection", "profile-file");
-    if(!url)
-    {
-        url = g_strdup_printf("%s/.gmpc/profiles.cfg", g_get_home_dir());
-        cfg_set_single_value_as_string(config, "connection", "profile-file",url);
-    }
-    profiles = cfg_open(url);
-    if(profiles == NULL)
-    {
-        /**
-         * Show gtk error message and quit 
-         */
-        debug_printf(DEBUG_ERROR,"Failed to save/load Profile file:\n%s\n",url);
-        show_error_message(_("Failed to load the configuration system"), TRUE);
-        abort();
-    }
-    q_free(url);
+
+	/**
+	 * TODO, Check if version changed, then say something about it 
+	 */
+	url = cfg_get_single_value_as_string(config, "Default", "version");
+	if(url == NULL || strcmp(url, VERSION))
+	{
+		printf("Welcome to a new version of gmpc.\n");
+		/* Do possible cleanup of config files and stuff */
+
+		/* set new */
+		cfg_set_single_value_as_string(config, "Default", "version",VERSION); 
+	}
+	if(url){
+		q_free(url);
+	}
+
+
+    gmpc_profiles = gmpc_profiles_new();
 
 
 #ifndef WIN32
@@ -514,117 +513,132 @@ int main (int argc, char **argv)
     /**
      *  load dynamic plugins 
      */
-    /** Load the global installed plugins */
-    url = g_strdup_printf("%s/%s",GLADE_PATH, "plugins");
-    plugin_load_dir(url);
-    q_free(url);
-    /* user space dynamic plugins */
-    url = g_strdup_printf("%s/.gmpc/plugins/",g_get_home_dir());
-    /**
-     * if dir exists, try to load the plugins.
-     */
-    if(g_file_test(url, G_FILE_TEST_IS_DIR))
-    {
-        plugin_load_dir(url);
-    }
-    q_free(url);
+#ifdef WIN32
+	packagedir = g_win32_get_package_installation_directory("gmpc", NULL);
+    debug_printf(DEBUG_INFO, "Got %s as package installation dir", packagedir);
+    url = g_build_filename(packagedir, "data", "plugins", NULL);
+	/* From a certain version of GTK+ this g_free will be needed, but for now it will free
+	 * a pointer which is returned on further calls to g_win32_get...
+	 * This bug is fixed now (30-10-2007), so it will probably be in glib 2.6.7 and/or 2.8.4
+	 */
+#if GLIB_CHECK_VERSION(2,8,4)
+	q_free(packagedir);
+#endif
 
 
-    /* time todo some initialisation of plugins */
-    for(i=0; i< num_plugins && plugins[i] != NULL;i++)
-    {
-        if(plugins[i]->init)
-        {
-            plugins[i]->init();
-        }
-    }
+#else
+	/** Load the global installed plugins */
+	url = g_strdup_printf("%s%c%s",GLADE_PATH,G_DIR_SEPARATOR, "plugins");
+#endif
+	plugin_load_dir(url);
+	q_free(url);
+	/* user space dynamic plugins */
+	url = g_strdup_printf("%s%c.gmpc%cplugins%c",g_get_home_dir(),G_DIR_SEPARATOR,G_DIR_SEPARATOR,G_DIR_SEPARATOR);
+	/**
+	 * if dir exists, try to load the plugins.
+	 */
+	if(g_file_test(url, G_FILE_TEST_IS_DIR))
+	{
+		plugin_load_dir(url);
+	}
+	q_free(url);
 
-    /**
-     * Create the backend store for the current playlist
-     */
-    init_playlist_store ();
 
-    /**
-     * Ask user about added/removed provider plugins 
-     */
-    meta_data_check_plugin_changed();
-    /**
-     * Create the main window
-     */	
-    debug_printf(DEBUG_INFO, "Create main window\n");
-    create_playlist3();
-    /** 
-     * If autoconnect is enabled, tell gmpc that it's in state it should connect
-     */	
-    if(cfg_get_single_value_as_int_with_default(config, "connection","autoconnect", DEFAULT_AUTOCONNECT))
-    {
-        gmpc_connected = TRUE;
-    }
-    /*
-     * create timeouts 
-     * get the status every 1/2 second should be enough, but it's configurable.
-     */
-    g_timeout_add (cfg_get_single_value_as_int_with_default(config,
-                "connection","mpd-update-speed",500),
-            (GSourceFunc)update_mpd_status, NULL);
-    /**
-     * create the autoconnect timeout, if autoconnect enable, it will check every 5 seconds
-     * if you are still connected, and reconnects you if not.
-     */
-    autoconnect_timeout = g_timeout_add (5000,(GSourceFunc)autoconnect_callback, NULL);
-    /** 
-     * Call this when entering the main loop, so you are connected on startup, not 5 seconds later
-     */
-    gtk_init_add((GSourceFunc)autoconnect_callback, NULL);
+	/* time todo some initialisation of plugins */
+	for(i=0; i< num_plugins && plugins[i] != NULL;i++)
+	{
+		if(plugins[i]->init)
+		{
+			plugins[i]->init();
+		}
+	}
+
+	/**
+	 * Create the backend store for the current playlist
+	 */
+	init_playlist_store ();
+
+	/**
+	 * Ask user about added/removed provider plugins 
+	 */
+	meta_data_check_plugin_changed();
+	/**
+	 * Create the main window
+	 */	
+	debug_printf(DEBUG_INFO, "Create main window\n");
+	create_playlist3();
+	/** 
+	 * If autoconnect is enabled, tell gmpc that it's in state it should connect
+	 */	
+	if(cfg_get_single_value_as_int_with_default(config, "connection","autoconnect", DEFAULT_AUTOCONNECT))
+	{
+		gmpc_connected = TRUE;
+	}
+	/*
+	 * create timeouts 
+	 * get the status every 1/2 second should be enough, but it's configurable.
+	 */
+	g_timeout_add (cfg_get_single_value_as_int_with_default(config,
+				"connection","mpd-update-speed",500),
+			(GSourceFunc)update_mpd_status, NULL);
+	/**
+	 * create the autoconnect timeout, if autoconnect enable, it will check every 5 seconds
+	 * if you are still connected, and reconnects you if not.
+	 */
+	autoconnect_timeout = g_timeout_add (5000,(GSourceFunc)autoconnect_callback, NULL);
+	/** 
+	 * Call this when entering the main loop, so you are connected on startup, not 5 seconds later
+	 */
+	gtk_init_add((GSourceFunc)autoconnect_callback, NULL);
 
 #ifdef ENABLE_MMKEYS
-    /**
-     * Setup Multimedia Keys
-     */
-    /**
-     * Create mmkeys object
-     */
-    keys = mmkeys_new();
-    /**
-     * Connect the wanted key's
-     */
-    g_signal_connect(G_OBJECT(keys), "mm_playpause", G_CALLBACK(play_song), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_next", G_CALLBACK(next_song), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_prev", G_CALLBACK(prev_song), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_stop", G_CALLBACK(stop_song), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_fastforward", G_CALLBACK(song_fastforward), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_fastbackward", G_CALLBACK(song_fastbackward), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_repeat", G_CALLBACK(repeat_toggle), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_random", G_CALLBACK(random_toggle), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_raise", G_CALLBACK(pl3_show_window), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_hide", G_CALLBACK(pl3_hide), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_toggle_hidden", G_CALLBACK(pl3_toggle_hidden), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_volume_up", G_CALLBACK(volume_up), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_volume_down", G_CALLBACK(volume_down), NULL);
-    g_signal_connect(G_OBJECT(keys), "mm_show_notification", G_CALLBACK(tray_notify_popup), NULL );
+	/**
+	 * Setup Multimedia Keys
+	 */
+	/**
+	 * Create mmkeys object
+	 */
+	keys = mmkeys_new();
+	/**
+	 * Connect the wanted key's
+	 */
+	g_signal_connect(G_OBJECT(keys), "mm_playpause", G_CALLBACK(play_song), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_next", G_CALLBACK(next_song), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_prev", G_CALLBACK(prev_song), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_stop", G_CALLBACK(stop_song), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_fastforward", G_CALLBACK(song_fastforward), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_fastbackward", G_CALLBACK(song_fastbackward), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_repeat", G_CALLBACK(repeat_toggle), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_random", G_CALLBACK(random_toggle), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_raise", G_CALLBACK(pl3_show_window), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_hide", G_CALLBACK(pl3_hide), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_toggle_hidden", G_CALLBACK(pl3_toggle_hidden), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_volume_up", G_CALLBACK(volume_up), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_volume_down", G_CALLBACK(volume_down), NULL);
+	g_signal_connect(G_OBJECT(keys), "mm_show_notification", G_CALLBACK(tray_notify_popup), NULL );
 
 
 #endif
 
 
-    /*
-     * run the main loop
-     */
-    gtk_main ();
+	/*
+	 * run the main loop
+	 */
+	gtk_main ();
 
-    /**
-     *  cleaning up. 
-     */
+	/**
+	 *  cleaning up. 
+	 */
 #ifndef WIN32
-    if(bacon_connection)
-    {
-        bacon_message_connection_free (bacon_connection);
-    }
+	if(bacon_connection)
+	{
+		bacon_message_connection_free (bacon_connection);
+	}
 #endif	
-    /**
-     * Clear metadata struct
-     */
-    meta_data_destroy();
+	/**
+	 * Clear metadata struct
+	 */
+	meta_data_destroy();
 
 	/* time todo some destruction of plugins */
 	for(i=0; i< num_plugins && plugins[i] != NULL;i++)
@@ -638,11 +652,13 @@ int main (int argc, char **argv)
 	 * Close the config file
 	 */
 	cfg_close(config);
-	cfg_close(profiles);
-    /** 
-     * Destroy the connection object 
-     */
-    mpd_free(connection);
+
+	g_object_unref(gmpc_profiles);
+	g_object_unref(gmpcconn);
+	/** 
+	 * Destroy the connection object 
+	 */
+	mpd_free(connection);
 
 	/**
 	 * remove (probly allready done) 
@@ -694,7 +710,7 @@ void main_quit()
 
 /**
  * Callback that get's called every 5 seconds, 
- * and trieds to autoconnect
+ * and tries to autoconnect
  * (when enabled)
  */
 static int autoconnect_callback(void)
@@ -724,154 +740,34 @@ static int autoconnect_callback(void)
 
 static void init_stock_icons()
 {
-	GtkIconFactory *factory;
-	GdkPixbuf *pb;
-	GtkIconSet *set;
 	char *path;
-	factory = gtk_icon_factory_new ();
+	GdkPixbuf *pb;
 
-	/*
-	 * add media-audiofile
-	 */
-	path = gmpc_get_full_image_path("media-audiofile.png");
-	pb = gdk_pixbuf_new_from_file (path ,NULL);
+	path = gmpc_get_full_image_path("");
+	gtk_icon_theme_append_search_path(gtk_icon_theme_get_default (),path);
 	q_free(path);
 
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "media-audiofile", set);
-	g_object_unref (G_OBJECT (pb));
-	/*
-	 * add media-stream
-	 */
-	path = gmpc_get_full_image_path("media-stream.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
-	q_free(path);
-
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "media-stream", set);
-	g_object_unref (G_OBJECT (pb));
-	/*
-	 * add media-artist
-	 */
-	path = gmpc_get_full_image_path("media-artist.png");
-	pb = gdk_pixbuf_new_from_file (path,NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "media-artist", set);
-	g_object_unref (G_OBJECT (pb));
-	/*
-	 * add media-album
-	 */
-	path = gmpc_get_full_image_path("media-album.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "media-album", set);
-	g_object_unref (G_OBJECT (pb));
-
-	/*
-	 * add player-shuffle
-	 */
-	path = gmpc_get_full_image_path("player-shuffle.png");
-	pb = gdk_pixbuf_new_from_file (path,NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "stock_shuffle", set);
-	g_object_unref (G_OBJECT (pb));
-	/*
-	 * add player-repeat
-	 */
-	path = gmpc_get_full_image_path("player-repeat.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "stock_repeat", set);
-	g_object_unref (G_OBJECT (pb));
-
-	/*
-	 * add media playlist
-	 */
-	path = gmpc_get_full_image_path("media-playlist.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "media-playlist", set);
-	g_object_unref (G_OBJECT (pb));
-
-
-	/*
-	 * add media playlist
-	 */
 	path = gmpc_get_full_image_path("gmpc.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
+	pb = gdk_pixbuf_new_from_file(path, NULL);
 	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "gmpc", set);
 	gtk_window_set_default_icon(pb);
-	g_object_unref (G_OBJECT (pb));
+	g_object_unref(G_OBJECT(pb));
 
-	/*
-	 * add media playlist
-	 */
-	path = gmpc_get_full_image_path("gmpc-tray.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
+#ifdef WIN32
+	/* hack to help files */
+	gchar *hack = NULL;
+	path = gmpc_get_full_image_path("");
+	hack = g_strdup_printf("%s%chicolor%c32x32%cactions",path, G_DIR_SEPARATOR,G_DIR_SEPARATOR, G_DIR_SEPARATOR);
+	gtk_icon_theme_append_search_path(gtk_icon_theme_get_default (),hack);
+	q_free(hack);
 	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "gmpc-tray", set);
-	g_object_unref (G_OBJECT (pb));
 
-	path = gmpc_get_full_image_path("gmpc-tray-play.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "gmpc-tray-play", set);
-	g_object_unref (G_OBJECT (pb));
+	/* The Windows gtkrc sets this to 0, so images don't work on buttons */
+	gtk_settings_set_long_property(gtk_settings_get_default(),
+			"gtk-button-images", TRUE, "main");
+#endif
 
-	path = gmpc_get_full_image_path("gmpc-tray-pause.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "gmpc-tray-pause", set);
-	g_object_unref (G_OBJECT (pb));
-
-
-	path = gmpc_get_full_image_path("gmpc-tray-disconnected.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);
-	gtk_icon_factory_add (factory, "gmpc-tray-disconnected", set);
-	g_object_unref (G_OBJECT (pb));
-
-	path = gmpc_get_full_image_path("gmpc-no-cover.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);              	
-	gtk_icon_factory_add (factory, "media-no-cover", set);
-	g_object_unref (G_OBJECT (pb));
-
-
-	path = gmpc_get_full_image_path("gmpc-loading-cover.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);              	
-	gtk_icon_factory_add (factory, "media-loading-cover", set);  	
-	g_object_unref (G_OBJECT (pb));
-
-	path = gmpc_get_full_image_path("stock_volume.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);              	
-	gtk_icon_factory_add (factory, "gmpc-volume", set);
-	g_object_unref (G_OBJECT (pb));
-
-	path = gmpc_get_full_image_path("gmpc-add-url.png");
-	pb = gdk_pixbuf_new_from_file (path, NULL);
-	q_free(path);
-	set = gtk_icon_set_new_from_pixbuf (pb);              	
-	gtk_icon_factory_add (factory, "gmpc-add-url", set);
-	g_object_unref (G_OBJECT (pb));
-
-	gtk_icon_factory_add_default (factory);
+	return;
 }
 
 /**
@@ -1193,7 +1089,7 @@ static void create_gmpc_paths(void)
 	 */	
 
 	/** create path */
-	gchar *url = g_strdup_printf("%s/.gmpc/", g_get_home_dir());
+	gchar *url = g_strdup_printf("%s%c.gmpc%c", g_get_home_dir(),G_DIR_SEPARATOR,G_DIR_SEPARATOR);
 	debug_printf(DEBUG_INFO, "Checking for %s existence",url);
 
 	/**
