@@ -29,6 +29,7 @@ enum PlaylistColums {
 	PL_COLUMN_ARTIST,
 	PL_COLUMN_TRACK,
 	PL_COLUMN_TITLEFILE,
+	PL_COLUMN_FILEBASENAME,
 	PL_COLUMN_ALBUM,	
 	PL_COLUMN_GENRE,
 	PL_COLUMN_COMPOSER,
@@ -36,8 +37,8 @@ enum PlaylistColums {
 	PL_COLUMN_LENGTH,
 	PL_COLUMN_DISC,
 	PL_COLUMN_COMMENT,
+	PL_COLUMN_DATE,
 	PL_COLUMN_SONGPOS,
 	PL_COLUMN_TOTAL
 };
-void pl3_update_profiles_menu(void);
 #endif
