@@ -37,7 +37,7 @@ void plugin_add(gmpcPlugin *plug, int plugin)
 static int plugin_load(char *path, const char *file)
 {
 	GModule *handle;
-	int *api_version;
+	int *api_version=0;
 	gmpcPlugin *plug = NULL;
 	gchar *string = NULL;
 	gchar *full_path = NULL;
@@ -45,7 +45,7 @@ static int plugin_load(char *path, const char *file)
 	{
 		return 1;
 	}
-	full_path = g_strdup_printf("%s/%s", path, file);
+	full_path = g_strdup_printf("%s%c%s", path,G_DIR_SEPARATOR, file);
 	debug_printf(DEBUG_INFO, "plugin_load: trying to load plugin %s", full_path);
 
 	handle = g_module_open(full_path, G_MODULE_BIND_LOCAL);
@@ -109,7 +109,7 @@ void plugin_load_dir(gchar *path)
 		const gchar *dirname = NULL;
 		while((dirname = g_dir_read_name(dir)) != NULL)
 		{
-			gchar *full_path = g_strdup_printf("%s/%s",path,dirname);
+			gchar *full_path = g_strdup_printf("%s%c%s",path,G_DIR_SEPARATOR,dirname);
 			if(g_file_test(full_path, G_FILE_TEST_IS_REGULAR))
 			{
 				if(plugin_load(path,dirname)){
