@@ -8,7 +8,7 @@
 
 #include "main.h"
 #include "playlist3.h"
-#include "playlist3-tag-browser.h"
+#include "browsers/playlist3-tag-browser.h"
 #include "config1.h"
 
 #define PLUGIN_STATS -200
@@ -283,8 +283,9 @@ static void plugin_stats_construct(GtkWidget *container)
 					case GMPC_PLUGIN_META_DATA:
 						gtk_list_store_set(store, &iter, 2, _("Metadata Provider"),-1);
 						break;
-
-
+          case GMPC_PLUGIN_PL_BROWSER|GMPC_PLUGIN_META_DATA:
+						gtk_list_store_set(store, &iter, 2, _("Metadata Provider and Browser Extention"),-1);
+            break;
 					case GMPC_PLUGIN_NO_GUI:
 						gtk_list_store_set(store, &iter, 2, _("Misc."),-1);
 						break;                                                         					
